<?php

require($_SERVER['DOCUMENT_ROOT'].'/modules/mail/vendor/PHPMailer/PHPMailerAutoload.php');

function gradeSendMail($subject, $body, $sendToEmail, $sendToName) {
    $mail = new PHPMailer;

    //$mail->SMTPDebug = 3;                               // Enable verbose debug output

    $mail->isSMTP();                                      // Set mailer to use SMTP
    $mail->Host = 'class.mmcs.sfedu.ru';  // Specify main and backup SMTP servers
    $mail->SMTPAuth = false;                               // Enable SMTP authentication
    $mail->Username = NULL;                 // SMTP username
    $mail->Password = NULL;                           // SMTP password
    $mail->SMTPSecure = NULL;                            // Enable TLS encryption, `ssl` also accepted
    $mail->Port = 25;                                    // TCP port to connect to

    $mail->From = 'no_reply@grade.sfedu.ru';
    $mail->FromName = 'Grade System (Сервис БРС)';
    $mail->addAddress($sendToEmail, $sendToName);     // Add a recipient
    //$mail->addAddress('ellen@example.com');               // Name is optional
    //$mail->addReplyTo('info@example.com', 'Information');
    //$mail->addCC('cc@example.com');
    //$mail->addBCC('bcc@example.com');

    //$mail->addAttachment('/var/tmp/file.tar.gz');         // Add attachments
    //$mail->addAttachment('/tmp/image.jpg', 'new.jpg');    // Optional name
    //$mail->isHTML(true);                                  // Set email format to HTML

    $mail->Subject = $subject;
    $mail->Body    = $body;
    //$mail->ContentType = 'text/plain';
    $mail->CharSet = 'utf-8';
    //$mail->AltBody = 'This is the body in plain text for non-HTML mail clients';

    if(!$mail->send()) {
        Log::instance()->add(Log::NOTICE, 'Message could not be sent: '.$mail->ErrorInfo);
        //echo 'Mailer Error: ' . $mail->ErrorInfo;
    } else {
        Log::instance()->add(Log::NOTICE, 'Message has been sent');
    }
}

class Kohana_Account {
    
    protected static $_instance;
    protected $_config;
    protected $_model;

    /**
     * Вовзращает экземпляр класса (singleton-паттерн)
     *
     * @return  instance
     */     
    public static function instance() {
        if(!isset(self::$_instance))
        {
            $config = Kohana::$config->load('account');
            self::$_instance = new self($config);
        }
        return self::$_instance;
    }
    
    private function __construct($config = array()) {
        $this->_config = $config;
        $this->_model = new Model_Account;
    }
    
    /**
     * Генерирует уникальный код активации для данных входных параметров.
     *
     * @param  string $FirstName Имя
     * @param  string $SecondName Отчество
     * @param  string $LastName Фамилия
     * @return  string
     */       
    protected function generateActivationCode()
    {
        $activationCode = Text::random('ABDCEFGHJKLMNPQRSTUVWXYZ123456789', 10);
        return UTF8::strtoupper($activationCode);
    }
    
    public function createTeacher($lastName, $firstName, $secondName, $degreeID, $departamentID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createTeacher( $lastName, $firstName,
                                                  $secondName, $degreeID,
                                                  $departamentID, $activationCode );
        if ($response === -1) {
            $activationCode = -1;
        }
        return $activationCode;
    }

    public function createTeacherByDepName($lastName, $firstName, $secondName, $departamentName, $facultyID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createTeacherByDepName( $lastName, $firstName,
                                                            $secondName, $departamentName, $facultyID,
                                                            $activationCode );
        if ($response === -1) {
            $activationCode = -1;
        }
        return $activationCode;
    }

    public function createStudent($lastName, $firstName, $secondName, $grade, $groupNum, $facultyID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createStudent( $lastName, $firstName, $secondName,
                                                  $grade, $groupNum, $facultyID,
                                                  $activationCode );
        if ($response === -1) {
            $activationCode = -1;
        }
        return $activationCode;
    }

    public function createStudentEx($lastName, $firstName, $secondName, $studentGradeNum, $studentGroupNum, $studentDegree, $studentSpec, $facultyID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createStudentEx($lastName, $firstName, $secondName, $studentGradeNum, $studentGroupNum, $studentDegree, $studentSpec, $facultyID, $activationCode);
        if ($response === -1) {
            $activationCode = -1;
        }
        return $activationCode;
    }

    public function createSubject($name, $abbr, $facultyID)
    {
        return $this->_model->createSubject($name, $abbr, $facultyID);
    }

    private function checkTokenLifetime($creationDate)
    {
        $config = Kohana::$config->load('security.securityPolicy');
        $lifetime = $config['recoveryToken']['lifetime'];
        return (time() - $creationDate) > $lifetime;
    }


    private function checkRecoveryStatus($email)
    {
        $recovery = $this->_model->GetRecoveryInfoByEMail($email);
        $status = true;
        foreach($recovery as $row) {
            $date = strtotime($row['Date']);
            if($this->checkTokenLifetime($date)) {
                $this->_model->useRecoveryToken($row['Token']);
            }
            //else {
            //    $status = false;
            //}
        }
        return $status;
    }

    public function checkToken($token)
    {
        $recovery = $this->_model->getRecoveryInfoByToken($token)->offsetGet(0);
        $response = true;
        
        if ($recovery['isUsed']) {
            $response = false;
        } else {
            $date = strtotime($recovery['Date']);
            if($this->checkTokenLifetime($date)) {
                $this->_model->useRecoveryToken($recovery['Token']);
                $response = false;
            }
        }
        return $response;
    }

    public function createRecoveryRequest($email)
    {
        $requestToken = sha1($email.time().Cookie::$salt);
        if($this->checkRecoveryStatus($email))
        {
            $this->_model->createRecoveryToken($email, $requestToken);
            $subject =  ASSEMBLY_SYSTEM_NAME.": Восстановление пароля";
            
            $twig = Twig::factory('email/recovery');
            $twig->curl = URL::base(TRUE, 'https');
            $twig->Token = $requestToken;
            $twig->EMail = $email;
            $twig->Subject = $subject;

            /* Kohana mailer is not working
			$status = Mailer::factory()
				->headers('Content-Type', 'text/html; charset=utf-8')
                ->subject($subject)
                ->in_reply_to(Mailer::message_id())
                ->from('no-reply@rating.mmcs.sfedu.ru')
                ->body($twig->render())
                ->send($email);*/
            gradeSendMail($subject, $twig->render(), $email, 'Username');
        }
    }

    public function changePasswordByToken($token, $password)
    {
        $recovery = $this->_model->getRecoveryInfoByToken($token)->offsetGet(0);
        $this->changePassword($recovery['AccountID'], $password);
        $this->_model->useRecoveryToken($token);
    }

    public function isLoginExists($login)
    {
        $login_count = $this->_model->getAccNumByLogin($login);
        return $login_count > 0;
    }

    public function isMailExists($email)
    {
        $email_count = $this->_model->getAccNumByMail($email);
        return $email_count > 0;
    }

    public function changeLogin($id, $newLogin)
    {
        $response = $this->_model->changeLogin($id, $newLogin);
        return $response != -1;
    }

    public function changeEMail($id, $newEMail)
    {
        $response = $this->_model->changeMail($id, $newEMail);
        return $response != -1;
    }

    public function changePassword($id, $newPassword)
    {
        $response = $this->_model->changePassword($id, $newPassword);
        return $response != -1;
    }
}